@@ -5,7 +5,7 @@
 Box of handy tools for Sphinx.
 """
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -26,6 +26,15 @@ Box of handy tools for Sphinx.
 #  OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
+# stdlib
+import sys
+
+# This all has to be up here so it's triggered first.
+if sys.version_info >= (3, 10):
+	# stdlib
+	import types
+	types.Union = types.UnionType
+
 # 3rd party
 from sphinx.application import Sphinx
 
@@ -47,19 +56,18 @@ from sphinx_toolbox.cache import cache  # noqa: F401
 
 __author__: str = "Dominic Davis-Foster"
 __copyright__: str = "2020 Dominic Davis-Foster"
-
 __license__: str = "MIT License"
-__version__: str = "2.0.3"
+__version__: str = "3.8.2"
 __email__: str = "dominic@davis-foster.co.uk"
 
-__all__ = ["setup"]
+__all__ = ("setup", )
 
 
 def setup(app: Sphinx) -> "utils.SphinxExtMetadata":
 	"""
 	Setup :mod:`sphinx_toolbox`.
 
-	:param app: The Sphinx app.
+	:param app: The Sphinx application.
 	"""
 
 	# Ensure dependencies are set up
@@ -70,14 +78,18 @@ def setup(app: Sphinx) -> "utils.SphinxExtMetadata":
 
 	# Setup standalone extensions
 	app.setup_extension("sphinx_toolbox.assets")
+	app.setup_extension("sphinx_toolbox.changeset")
 	app.setup_extension("sphinx_toolbox.code")
+	app.setup_extension("sphinx_toolbox.collapse")
 	app.setup_extension("sphinx_toolbox.confval")
 	app.setup_extension("sphinx_toolbox.decorators")
 	app.setup_extension("sphinx_toolbox.formatting")
 	app.setup_extension("sphinx_toolbox.installation")
 	app.setup_extension("sphinx_toolbox.issues")
+	app.setup_extension("sphinx_toolbox.latex")
 	app.setup_extension("sphinx_toolbox.rest_example")
 	app.setup_extension("sphinx_toolbox.shields")
+	app.setup_extension("sphinx_toolbox.sidebar_links")
 	app.setup_extension("sphinx_toolbox.source")
 	app.setup_extension("sphinx_toolbox.wikipedia")
 	app.setup_extension("sphinx_toolbox.more_autodoc.autoprotocol")
