@@ -5,9 +5,13 @@
 Adds the ``:no-docstring:`` option to automodule directives to exclude the docstring from the output.
 
 .. versionadded:: 1.0.0
+
+	Functions in this module moved from ``sphinx_toolbox.more_autodoc.__init__.py``
+
+.. extensions:: sphinx_toolbox.more_autodoc.no_docstring
 """
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -33,55 +37,51 @@ from types import ModuleType
 from typing import List
 
 # 3rd party
-import autodocsumm  # type: ignore
+import autodocsumm  # type: ignore[import-untyped]
 import sphinx.ext.autodoc
 from sphinx.application import Sphinx
 
 # this package
-from sphinx_toolbox.utils import SphinxExtMetadata, flag
+from sphinx_toolbox.more_autosummary import PatchedAutoSummModuleDocumenter
+from sphinx_toolbox.utils import SphinxExtMetadata, flag, metadata_add_version
 
-__all__ = ["automodule_add_nodocstring", "no_docstring_process_docstring", "setup"]
+__all__ = ("automodule_add_nodocstring", "no_docstring_process_docstring", "setup")
 
 
-def automodule_add_nodocstring(app) -> None:
+def automodule_add_nodocstring(app: Sphinx) -> None:
 	"""
-	Add the ``:no-docstring:`` option to automodule directives to exclude the docstring from the output.
+	Add the ``:no-docstring:`` option to automodule directives.
 
-	:param app: The Sphinx app.
+	The option is used to exclude the docstring from the output
 
-	.. versionchanged:: 1.0.0
-
-		Moved from ``sphinx_toolbox.more_autodoc.__init__.py``
+	:param app: The Sphinx application.
 	"""
 
 	sphinx.ext.autodoc.ModuleDocumenter.option_spec["no-docstring"] = flag
 	autodocsumm.AutoSummModuleDocumenter.option_spec["no-docstring"] = flag
+	PatchedAutoSummModuleDocumenter.option_spec["no-docstring"] = flag
 
 	app.setup_extension("sphinx.ext.autodoc")
 	app.connect("autodoc-process-docstring", no_docstring_process_docstring, priority=1000)
 
 
-def no_docstring_process_docstring(
+def no_docstring_process_docstring(  # noqa: MAN001
 		app: Sphinx,
 		what,
 		name: str,
 		obj,
 		options,
 		lines: List[str],
-		):
+		) -> None:
 	"""
-	Process the docstring of a module, and remove its docstring of the ``:no-docstring:`` flag was set..
+	Remove module docstrings if the ``:no-docstring:`` flag was set.
 
-	:param app: The Sphinx app
+	:param app: The Sphinx application.
 	:param what:
-	:param name: The name of the object being documented
+	:param name: The name of the object being documented.
 	:param obj: The object being documented.
 	:param options: Mapping of autodoc options to values.
 	:param lines: List of strings representing the current contents of the docstring.
-
-	.. versionchanged:: 1.0.0
-
-		Moved from ``sphinx_toolbox.more_autodoc.__init__.py``
 	"""
 
 	if isinstance(obj, ModuleType):
@@ -90,19 +90,14 @@ def no_docstring_process_docstring(
 			lines.clear()
 
 
+@metadata_add_version
 def setup(app: Sphinx) -> SphinxExtMetadata:
 	"""
 	Setup :mod:`sphinx_toolbox.more_autodoc.no_docstring`.
 
-	:param app: The Sphinx app.
+	:param app: The Sphinx application.
 	"""
 
-	# this package
-	from sphinx_toolbox import __version__
-
 	automodule_add_nodocstring(app)
 
-	return {
-			"version": __version__,
-			"parallel_read_safe": True,
-			}
+	return {"parallel_read_safe": True}
