@@ -11,7 +11,7 @@ The changes are:
   If `intersphinx <https://www.sphinx-doc.org/en/master/usage/extensions/intersphinx.html>`_
   is used this will now be a link to the Python documentation.
 
-  Since :pull:`154 <agronholm/sphinx-autodoc-typehints>` this feature is now available upstream.
+  Since :github:pull:`154 <tox-dev/sphinx-autodoc-typehints>` this feature is now available upstream.
 
 * If the signature of the object cannot be read, the signature provided by Sphinx will be used
   rather than raising an error.
@@ -21,7 +21,7 @@ The changes are:
 * :class:`typing.TypeVar`\s are linked to if they have been included in the documentation.
 
 * If a function/method argument has a :class:`module <types.ModuleType>`, :class:`class <typing.Type>`
-  or :class:`function <types.FunctionType>` object as its default value a better
+  or :py:obj:`function <types.FunctionType>` object as its default value a better
   representation will be shown in the signature.
 
   For example:
@@ -46,11 +46,13 @@ The changes are:
           docstring_hooks.append((my_hook, 75))
           return {}
 
+  .. latex:clearpage::
+
   ``my_hook`` is a function that takes the object being documented as its only argument
   and returns that object after modification. The ``75`` is the priority of the hook:
 
    * ``< 20`` runs before ``fget`` functions are extracted from properties
-   * ``< 90`` runs before ``__new__`` functions are extracted from :class:`NamedTuples <typing.NamedTuple>`.
+   * ``< 90`` runs before ``__new__`` functions are extracted from :class:`~typing.NamedTuple`\s.
    * ``< 100`` runs before ``__init__`` functions are extracted from classes.
 
 * Unresolved forward references are handled better.
@@ -59,12 +61,20 @@ The changes are:
 
 -----
 
+.. versionadded:: 0.4.0
+.. versionchanged:: 0.6.0  Moved from :mod:`sphinx_toolbox.autodoc_typehints`.
+.. versionchanged:: 0.8.0  Added support for :func:`collections.namedtuple`.
+
+-----
+
 .. extensions:: sphinx_toolbox.more_autodoc.typehints
 
+.. latex:vspace:: -20px
 
-For configuration information see https://github.com/agronholm/sphinx-autodoc-typehints
+| For configuration information see https://github.com/agronholm/sphinx-autodoc-typehints
+| In addition, the following configuration value is added by this extension:
 
-In addition the following configuration value is added by this extension:
+.. latex:vspace:: -20px
 
 .. confval:: hide_none_rtype
 	:type: :class:`bool`
@@ -73,20 +83,13 @@ In addition the following configuration value is added by this extension:
 	Hides return types of :py:obj:`None`.
 
 
-.. versionadded:: 0.4.0
+API Reference
+-----------------
 
-.. versionchanged:: 0.6.0
-
-	Moved from :mod:`sphinx_toolbox.autodoc_typehints`.
-
-.. versionchanged:: 0.8.0
-
-	Added support for namedtuples.
-
-"""  # noqa SXL001
+"""  # noqa: SXL001
 #
 #  Copyright (c) Alex Grönholm
-#  Changes copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Changes copyright © 2020-2022 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -111,21 +114,34 @@ In addition the following configuration value is added by this extension:
 import inspect
 import itertools
 import json
-import operator
 import re
 import sys
 import types
+from contextlib import suppress
+from operator import itemgetter
+from tempfile import TemporaryDirectory
 from types import FunctionType, ModuleType
-from typing import Any, AnyStr, Callable, Dict, List, Optional, Tuple, Type, TypeVar, get_type_hints
-
-if sys.version_info[:2] == (3, 6):
-	# stdlib
-	from typing import _ForwardRef as ForwardRef  # type: ignore
-else:
-	from typing import ForwardRef
+from typing import (
+		Any,
+		AnyStr,
+		Callable,
+		Dict,
+		ForwardRef,
+		List,
+		Mapping,
+		NewType,
+		Optional,
+		Tuple,
+		Type,
+		TypeVar,
+		Union,
+		get_type_hints
+		)
 
 # 3rd party
-import sphinx_autodoc_typehints  # type: ignore
+import sphinx.util.typing
+import sphinx_autodoc_typehints
+from domdf_python_tools.stringlist import DelimitedList
 from domdf_python_tools.typing import (
 		ClassMethodDescriptorType,
 		MethodDescriptorType,
@@ -137,6 +153,7 @@ from sphinx.application import Sphinx
 from sphinx.errors import ExtensionError
 from sphinx.util.inspect import signature as Signature
 from sphinx.util.inspect import stringify_signature
+from typing_extensions import Self
 
 # this package
 from sphinx_toolbox.utils import (
@@ -155,7 +172,37 @@ except ImportError:  # pragma: no cover
 	# where it is available to import, so it is fine to do this.
 	pass
 
-__all__ = [
+try:
+	# 3rd party
+	from sphinx_autodoc_typehints import logger as sat_logger  # type: ignore[attr-defined]
+	from sphinx_autodoc_typehints import pydata_annotations  # type: ignore[attr-defined]
+except ImportError:
+	# Privatised in 1.14.1
+	# 3rd party
+	from sphinx_autodoc_typehints import _LOGGER as sat_logger
+	from sphinx_autodoc_typehints import _PYDATA_ANNOTATIONS as pydata_annotations
+
+try:
+	# 3rd party
+	from sphinx_autodoc_typehints import _future_annotations_imported
+except ImportError:
+
+	def _future_annotations_imported(obj: Any) -> bool:
+		if sys.version_info < (3, 7):  # pragma: no cover (py37+)
+			# Only Python ≥ 3.7 supports PEP563.
+			return False
+
+		_annotations = getattr(inspect.getmodule(obj), "annotations", None)
+		if _annotations is None:
+			return False
+
+		# Make sure that annotations is imported from __future__ - defined in cpython/Lib/__future__.py
+		# annotations become strings at runtime
+		CO_FUTURE_ANNOTATIONS = 0x100000 if sys.version_info[0:2] == (3, 7) else 0x1000000
+		return _annotations.compiler_flag == CO_FUTURE_ANNOTATIONS
+
+
+__all__ = (
 		"ObjectAlias",
 		"Module",
 		"Function",
@@ -172,10 +219,9 @@ __all__ = [
 		"backfill_type_hints",
 		"load_args",
 		"split_type_comment_args",
-		"builder_ready",
 		"default_preprocessors",
 		"Preprocessor",
-		]
+		)
 
 get_annotation_module = sphinx_autodoc_typehints.get_annotation_module
 get_annotation_class_name = sphinx_autodoc_typehints.get_annotation_class_name
@@ -183,11 +229,10 @@ get_annotation_args = sphinx_autodoc_typehints.get_annotation_args
 backfill_type_hints = sphinx_autodoc_typehints.backfill_type_hints
 load_args = sphinx_autodoc_typehints.load_args
 split_type_comment_args = sphinx_autodoc_typehints.split_type_comment_args
-builder_ready = sphinx_autodoc_typehints.builder_ready
 
 
 # Demonstration of module default argument in signature
-def serialise(obj: Any, library=json) -> str:
+def serialise(obj: Any, library=json) -> str:  # type: ignore[empty-body]  # noqa: MAN001
 	"""
 	Serialise an object into a JSON string.
 
@@ -203,9 +248,9 @@ class ObjectAlias:
 	"""
 	Used to represent a module, class, function etc in a Sphinx function/class signature.
 
-	:param name: The name of the object being aliased.
-
 	.. versionadded:: 0.9.0
+
+	:param name: The name of the object being aliased.
 	"""
 
 	_alias_type: str
@@ -235,9 +280,9 @@ class Function(ObjectAlias):
 	"""
 	Used to represent a function in a Sphinx function/class signature.
 
-	:param name: The name of the function.
-
 	.. versionadded:: 0.9.0
+
+	:param name: The name of the function.
 	"""
 
 	_alias_type = "function"
@@ -247,20 +292,27 @@ class Class(ObjectAlias):
 	"""
 	Used to represent a class in a Sphinx function/class signature.
 
-	:param name: The name of the class.
-
 	.. versionadded:: 0.9.0
+
+	:param name: The name of the class.
 	"""
 
 	_alias_type = "class"
 
 
-def format_annotation(annotation, fully_qualified: bool = False) -> str:
+def format_annotation(annotation: Any, fully_qualified: bool = False) -> str:
 	"""
 	Format a type annotation.
 
 	:param annotation:
-	:param fully_qualified:
+	:param fully_qualified: Whether the fully qualified name should be shown (e.g. ``typing.List``)
+		or only the object name (e.g. ``List``).
+
+	:rtype:
+
+	.. versionchanged:: 2.13.0  Added support for :py:obj:`True` and :py:obj:`False`
+
+	.. latex:clearpage::
 	"""
 
 	prefix = '' if fully_qualified else '~'
@@ -268,80 +320,109 @@ def format_annotation(annotation, fully_qualified: bool = False) -> str:
 	# Special cases
 	if annotation is None or annotation is type(None):  # noqa: E721
 		return ":py:obj:`None`"
+	elif isinstance(annotation, bool):
+		return f":py:obj:`{annotation}`"
 	elif annotation is Ellipsis:
 		return "..."
+	elif annotation is Self:
+		return f":py:obj:`{prefix}typing.Self`"
 	elif annotation is itertools.cycle:
-		return ":func:`itertools.cycle`"
-	elif annotation is types.GetSetDescriptorType:  # noqa E721
+		return f":func:`{prefix}itertools.cycle`"
+	elif annotation is types.GetSetDescriptorType:  # noqa: E721
 		return f":py:data:`{prefix}types.GetSetDescriptorType`"
-	elif annotation is types.MemberDescriptorType:  # noqa E721
+	elif annotation is types.MemberDescriptorType:  # noqa: E721
 		return f":py:data:`{prefix}types.MemberDescriptorType`"
-	elif annotation is types.MappingProxyType:  # noqa E721
+	elif annotation is types.MappingProxyType:  # noqa: E721
 		return f":py:class:`{prefix}types.MappingProxyType`"
-	elif annotation is types.ModuleType:  # noqa E721
+	elif annotation is types.ModuleType:  # noqa: E721
 		return f":py:class:`{prefix}types.ModuleType`"
-	elif annotation is ClassMethodDescriptorType:  # noqa E721
-		return f":py:data:`{prefix}types.ClassMethodDescriptorType`"
-	elif annotation is MethodDescriptorType:  # noqa E721
-		return f":py:data:`{prefix}types.MethodDescriptorType`"
-	elif annotation is MethodWrapperType:  # noqa E721
-		return f":py:data:`{prefix}types.MethodWrapperType`"
-	elif annotation is WrapperDescriptorType:  # noqa E721
-		return f":py:data:`{prefix}types.WrapperDescriptorType`"
-	elif annotation is types.BuiltinFunctionType:  # noqa E721
+	elif annotation is types.FunctionType:  # noqa: E721
+		return f":py:data:`{prefix}types.FunctionType`"
+	elif annotation is types.BuiltinFunctionType:  # noqa: E721
 		return f":py:data:`{prefix}types.BuiltinFunctionType`"
-	elif annotation is types.MethodType:  # noqa E721
+	elif annotation is types.MethodType:  # noqa: E721
 		return f":py:data:`{prefix}types.MethodType`"
+	elif annotation is MethodDescriptorType:
+		return f":py:data:`{prefix}types.MethodDescriptorType`"
+	elif annotation is ClassMethodDescriptorType:
+		return f":py:data:`{prefix}types.ClassMethodDescriptorType`"
+	elif annotation is MethodWrapperType:
+		return f":py:data:`{prefix}types.MethodWrapperType`"
+	elif annotation is WrapperDescriptorType:
+		return f":py:data:`{prefix}types.WrapperDescriptorType`"
 	elif isinstance(annotation, ForwardRef):
 		# Unresolved forward ref
 		return f":py:obj:`{prefix}.{annotation.__forward_arg__}`"
-	elif annotation is type(re.compile('')):  # noqa E721
+	elif annotation is type(re.compile('')):  # noqa: E721
 		return f":py:class:`{prefix}typing.Pattern`"
+	elif annotation is TemporaryDirectory:
+		return f":py:obj:`{prefix}tempfile.TemporaryDirectory`"
+	elif sys.version_info >= (3, 10):  # pragma: no cover (<py310)
+		if annotation is types.UnionType:  # noqa: E721
+			return f":py:data:`{prefix}types.UnionType`"
 
 	try:
 		module = get_annotation_module(annotation)
 		class_name = get_annotation_class_name(annotation, module)
+
+		# Special case for typing.NewType being a class in 3.10
+		# Fixed upstream in 1.13.0
+		if sys.version_info >= (3, 10) and isinstance(annotation, NewType):  # pragma: no cover (<py310)
+			module, class_name = "typing", "NewType"
+
 		args = get_annotation_args(annotation, module, class_name)
 	except ValueError:
 		return f":py:obj:`~.{annotation}`"
 
 	if module == "_io":
 		module = "io"
-
+	elif module == "_ast":
+		module = "ast"
 	# Redirect all typing_extensions types to the stdlib typing module
-	if module == "typing_extensions":
+	elif module == "typing_extensions":
 		module = "typing"
+	elif module == "typing" and class_name == "AbstractContextManager":
+		module = "contextlib"
 
-	full_name = (module + '.' + class_name) if module != "builtins" else class_name
+	full_name = (f"{module}.{class_name}") if module != "builtins" else class_name
 	prefix = '' if fully_qualified or full_name == class_name else '~'
-	role = "data" if class_name in sphinx_autodoc_typehints.pydata_annotations else "class"
+	role = "data" if class_name in pydata_annotations else "class"
 	args_format = "\\[{}]"
 	formatted_args = ''
 
 	# Type variables are also handled specially
-	try:
+	with suppress(TypeError):
 		if isinstance(annotation, TypeVar) and annotation is not AnyStr:
-			if sys.version_info < (3, 7):
-				typevar_name = annotation.__name__
-			else:
-				typevar_name = (annotation.__module__ + '.' + annotation.__name__)
+			typevar_name = (annotation.__module__ + '.' + annotation.__name__)
 			return f":py:data:`{repr(annotation)} <{typevar_name}>`"
-	except TypeError:
-		pass
 
 	# Some types require special handling
 	if full_name == "typing.NewType":
 		args_format = f"\\(:py:data:`~{annotation.__name__}`, {{}})"
-		role = "func"
-	elif full_name == "typing.Union" and len(args) == 2 and type(None) in args:
+		role = "class" if sys.version_info > (3, 10) else "func"
+	elif full_name in {"typing.Union", "types.UnionType"} and len(args) == 2 and type(None) in args:
 		full_name = "typing.Optional"
-		args = tuple(x for x in args if x is not type(None))  # noqa: E721
+	elif full_name == "types.UnionType":
+		full_name = "typing.Union"
+		role = "data"
 	elif full_name == "typing.Callable" and args and args[0] is not ...:
 		formatted_args = "\\[\\[" + ", ".join(format_annotation(arg) for arg in args[:-1]) + ']'
 		formatted_args += ", " + format_annotation(args[-1]) + ']'
 	elif full_name == "typing.Literal":
-		# TODO: Bool, Enums?
-		formatted_args = "\\[" + ", ".join(code_repr(arg) for arg in args) + ']'
+		# TODO: Enums?
+		formatted_arg_list: DelimitedList[str] = DelimitedList()
+		for arg in args:
+			if isinstance(arg, bool):
+				formatted_arg_list.append(format_annotation(arg))
+			else:
+				formatted_arg_list.append(code_repr(arg))
+
+		formatted_args = f"\\[{formatted_arg_list:, }]"
+
+	if full_name == "typing.Optional":
+		args = tuple(x for x in args if x is not type(None))  # noqa: E721
+
+	# TODO: unions with one or more forward refs
 
 	if args and not formatted_args:
 		formatted_args = args_format.format(", ".join(format_annotation(arg, fully_qualified) for arg in args))
@@ -369,11 +450,11 @@ def preprocess_function_defaults(obj: Callable) -> Tuple[Optional[inspect.Signat
 	"""
 	Pre-processes the default values for the arguments of a function.
 
+	.. versionadded:: 0.8.0
+
 	:param obj: The function.
 
 	:return: The function signature and a list of arguments/parameters.
-
-	.. versionadded:: 0.8.0
 	"""
 
 	try:
@@ -382,17 +463,20 @@ def preprocess_function_defaults(obj: Callable) -> Tuple[Optional[inspect.Signat
 		return None, []
 
 	parameters = []
+	preprocessor_list = default_preprocessors
 
 	for param in signature.parameters.values():
 		default = param.default
 
+		# pylint: disable=dotted-import-in-loop
 		if default is not inspect.Parameter.empty:
-			for check, preprocessor in default_preprocessors:
+			for check, preprocessor in preprocessor_list:
 				if check(default):
 					default = preprocessor(default)
 					break
 
 		parameters.append(param.replace(annotation=inspect.Parameter.empty, default=default))
+		# pylint: enable=dotted-import-in-loop
 
 	return signature, parameters
 
@@ -403,30 +487,31 @@ def preprocess_class_defaults(
 	"""
 	Pre-processes the default values for the arguments of a class.
 
+	.. versionadded:: 0.8.0
+
 	:param obj: The class.
 
-	:return: The class signature and a list of arguments/parameters
-
-	.. versionadded:: 0.8.0
+	:return: The class signature and a list of arguments/parameters.
 	"""
 
-	init = getattr(obj, "__init__", getattr(obj, "__new__", None))
+	init: Optional[Callable[..., Any]] = getattr(obj, "__init__", getattr(obj, "__new__", None))
 
 	if is_namedtuple(obj):
 		init = getattr(obj, "__new__")
 
 	try:
-		signature = Signature(inspect.unwrap(init))
+		signature = Signature(inspect.unwrap(init))  # type: ignore[arg-type]
 	except ValueError:  # pragma: no cover
 		return init, None, []
 
 	parameters = []
+	preprocessor_list = default_preprocessors
 
 	for argname, param in signature.parameters.items():
 		default = param.default
 
 		if default is not inspect.Parameter.empty:
-			for check, preprocessor in default_preprocessors:
+			for check, preprocessor in preprocessor_list:
 				if check(default):
 					default = preprocessor(default)
 					break
@@ -434,16 +519,17 @@ def preprocess_class_defaults(
 			else:
 				if hasattr(obj, "__attrs_attrs__") and default is attr.NOTHING:
 					# Special casing for attrs classes
-					for value in obj.__attrs_attrs__:  # type: ignore
-						if value.name == argname and isinstance(value.default, attr.Factory):  # type: ignore
-							default = value.default.factory()
+					for value in obj.__attrs_attrs__:
+						if value.name == argname:
+							if isinstance(value.default, attr.Factory):  # type: ignore[arg-type]
+								default = value.default.factory()
 
 		parameters.append(param.replace(annotation=inspect.Parameter.empty, default=default))
 
 	return init, signature, parameters
 
 
-def process_signature(
+def process_signature(  # noqa: MAN001
 		app: Sphinx,
 		what: str,
 		name: str,
@@ -455,7 +541,7 @@ def process_signature(
 	"""
 	Process the signature for a function/method.
 
-	:param app: The Sphinx app
+	:param app: The Sphinx application.
 	:param what:
 	:param name: The name of the object being documented.
 	:param obj: The object being documented.
@@ -503,7 +589,7 @@ def process_signature(
 			hasattr(obj, "__qualname__") and "<locals>" in obj.__qualname__
 			and not _is_dataclass(name, what, obj.__qualname__)
 			):
-		sphinx_autodoc_typehints.logger.warning(
+		sat_logger.warning(
 				"Cannot treat a function defined as a local function: '%s'  (use @functools.wraps)", name
 				)
 		return None
@@ -597,9 +683,9 @@ def process_docstring(
 	"""
 	Process the docstring of a class, function, method etc.
 
-	:param app: The Sphinx app
+	:param app: The Sphinx application.
 	:param what:
-	:param name: The name of the object being documented
+	:param name: The name of the object being documented.
 	:param obj: The object being documented.
 	:param options: Mapping of autodoc options to values.
 	:param lines: List of strings representing the current contents of the docstring.
@@ -609,24 +695,42 @@ def process_docstring(
 		An empty ``:rtype:`` flag can be used to control the position of the return type annotation in the docstring.
 	"""
 
+	if what in {"variable", "regex"}:
+		# Doesn't have parameters or return type
+		return
+
 	original_obj = obj
 
-	for hook, priority in sorted(docstring_hooks, key=operator.itemgetter(1)):
+	for hook, priority in sorted(docstring_hooks, key=itemgetter(1)):
 		obj = hook(obj)
 
 	if callable(obj):
 		obj = inspect.unwrap(obj)
 		type_hints = get_all_type_hints(obj, name, original_obj)
 
+		signature_params: Mapping[str, Any]
+		try:
+			signature_params = inspect.signature(obj).parameters
+		except Exception:
+			# Ignore errors. Can be a multitude of things including builtin functions or extension modules.
+			signature_params = {}
+
 		for argname, annotation in type_hints.items():
 			if argname == "return":
 				continue  # this is handled separately later
 
+			if argname in signature_params:
+				# Ensure *args and **kwargs have *s
+				if signature_params[argname].kind == 2:  # *args
+					argname = f"\\*{argname}"
+				elif signature_params[argname].kind == 4:  # **kwargs
+					argname = f"\\*\\*{argname}"
+
 			argname = escape_trailing__(argname)
 
 			formatted_annotation = format_annotation(
 					annotation,
-					fully_qualified=app.config.typehints_fully_qualified,  # type: ignore
+					fully_qualified=app.config.typehints_fully_qualified,
 					)
 
 			searchfor = [f":{field} {argname}:" for field in ("param", "parameter", "arg", "argument")]
@@ -637,7 +741,7 @@ def process_docstring(
 					insert_index = i
 					break
 
-			if insert_index is None and app.config.always_document_param_types:  # type: ignore
+			if insert_index is None and app.config.always_document_param_types:
 				lines.append(f":param {argname}:")
 				insert_index = len(lines)
 
@@ -651,7 +755,7 @@ def process_docstring(
 
 			formatted_annotation = format_annotation(
 					type_hints["return"],
-					fully_qualified=app.config.typehints_fully_qualified,  # type: ignore
+					fully_qualified=app.config.typehints_fully_qualified,
 					)
 
 			insert_index = len(lines)
@@ -668,7 +772,7 @@ def process_docstring(
 				elif line.startswith(":return:") or line.startswith(":returns:"):
 					insert_index = i
 
-			if insert_index is not None and app.config.typehints_document_rtype:  # type: ignore
+			if insert_index is not None and app.config.typehints_document_rtype:
 
 				if insert_index == len(lines):
 					# Ensure that :rtype: doesn't get joined with a paragraph of text, which
@@ -676,36 +780,11 @@ def process_docstring(
 					lines.append('')
 					insert_index += 1
 
-				if not (formatted_annotation == ":py:obj:`None`" and app.config.hide_none_rtype):  # type: ignore
+				if not (formatted_annotation == ":py:obj:`None`" and app.config.hide_none_rtype):
 					lines.insert(insert_index, f":rtype: {formatted_annotation}")
 
 
-@metadata_add_version
-def setup(app: Sphinx) -> SphinxExtMetadata:
-	"""
-	Setup :mod:`sphinx_toolbox.more_autodoc.typehints`.
-
-	:param app: The Sphinx app
-	"""
-
-	if "sphinx_autodoc_typehints" in app.extensions:
-		raise ExtensionError(
-				"'sphinx_toolbox.more_autodoc.typehints' must be loaded before 'sphinx_autodoc_typehints."
-				)
-
-	sphinx_autodoc_typehints.format_annotation = format_annotation
-	sphinx_autodoc_typehints.process_signature = process_signature
-	sphinx_autodoc_typehints.process_docstring = process_docstring
-
-	app.setup_extension("sphinx.ext.autodoc")
-	app.setup_extension("sphinx_autodoc_typehints")
-
-	app.add_config_value("hide_none_rtype", False, "env", [bool])
-
-	return {"parallel_read_safe": True}
-
-
-def _class_get_type_hints(obj, globalns=None, localns=None):
+def _class_get_type_hints(obj, globalns=None, localns=None):  # noqa: MAN001,MAN002
 	"""
 	Return type hints for an object.
 
@@ -723,7 +802,7 @@ def _class_get_type_hints(obj, globalns=None, localns=None):
 
 	while True:
 
-		try:
+		try:  # pylint: disable=R8203
 			return get_type_hints(obj.__init__, localns=localns, globalns=globalns)
 		except NameError:
 			if not mro_stack:
@@ -734,7 +813,7 @@ def _class_get_type_hints(obj, globalns=None, localns=None):
 			localns = {**sys.modules[klasse.__module__].__dict__, **localns}
 
 
-def get_all_type_hints(obj, name, original_obj):
+def get_all_type_hints(obj: Any, name: str, original_obj: Any) -> Dict[str, Any]:
 	"""
 	Returns the resolved type hints for the given objects.
 
@@ -743,8 +822,8 @@ def get_all_type_hints(obj, name, original_obj):
 	:param original_obj: The original object, before the class if ``obj`` is its ``__init__`` method.
 	"""
 
-	def log(exc):
-		sphinx_autodoc_typehints.logger.warning(
+	def log(exc: Exception) -> None:
+		sat_logger.warning(
 				'Cannot resolve forward reference in type annotations of "%s": %s',
 				name,
 				exc,
@@ -757,10 +836,19 @@ def get_all_type_hints(obj, name, original_obj):
 			rv = _class_get_type_hints(original_obj)
 		else:
 			rv = get_type_hints(obj)
-	except (AttributeError, TypeError, RecursionError):
+
+	except (AttributeError, TypeError, RecursionError) as exc:
 		# Introspecting a slot wrapper will raise TypeError, and some recursive type
 		# definitions will cause a RecursionError (https://github.com/python/typing/issues/574).
-		pass
+
+		# If one is using PEP563 annotations, Python will raise a (e.g.,)
+		# TypeError("TypeError: unsupported operand type(s) for |: 'type' and 'NoneType'")
+		# on 'str | None', therefore we accept TypeErrors with that error message
+		# if 'annotations' is imported from '__future__'.
+		if isinstance(exc, TypeError):
+			if _future_annotations_imported(obj) and "unsupported operand type" in str(exc):
+				rv = obj.__annotations__
+
 	except NameError as exc:
 		log(exc)
 		rv = obj.__annotations__
@@ -788,3 +876,52 @@ def get_all_type_hints(obj, name, original_obj):
 		rv = obj.__annotations__
 
 	return rv
+
+
+@metadata_add_version
+def setup(app: Sphinx) -> SphinxExtMetadata:
+	"""
+	Setup :mod:`sphinx_toolbox.more_autodoc.typehints`.
+
+	:param app: The Sphinx application.
+	"""
+
+	if "sphinx_autodoc_typehints" in app.extensions:
+		raise ExtensionError(
+				"'sphinx_toolbox.more_autodoc.typehints' must be loaded before 'sphinx_autodoc_typehints'."
+				)
+
+	sphinx_autodoc_typehints.format_annotation = format_annotation  # type: ignore[assignment]
+	sphinx_autodoc_typehints.process_signature = process_signature
+	sphinx_autodoc_typehints.process_docstring = process_docstring  # type: ignore[assignment]
+
+	app.setup_extension("sphinx.ext.autodoc")
+	app.setup_extension("sphinx_autodoc_typehints")
+
+	app.add_config_value("hide_none_rtype", False, "env", [bool])
+
+	return {"parallel_read_safe": True}
+
+
+def _resolve_forwardref(
+		fr: Union[ForwardRef, sphinx.util.typing.ForwardRef],
+		module: str,
+		) -> object:
+	"""
+	Resolve a forward reference.
+
+	This is not part of the public API.
+
+	:param fr:
+	:param module: The module the forward reference was defined in.
+
+	:return: The evaluated object.
+	"""
+
+	module_dict = sys.modules[module].__dict__
+	if sys.version_info < (3, 9):
+		return fr._evaluate(module_dict, module_dict)
+	elif sys.version_info >= (3, 12):
+		return fr._evaluate(module_dict, module_dict, set(), recursive_guard=set())
+	else:
+		return fr._evaluate(module_dict, module_dict, set())
