@@ -5,17 +5,12 @@ r"""
 Documenter for module level variables, similar to :rst:dir:`autodata` but
 with a different appearance and more customisation options.
 
-.. extensions:: sphinx_toolbox.more_autodoc.variables
-
 .. versionadded:: 0.6.0
+.. extensions:: sphinx_toolbox.more_autodoc.variables
+.. versionchanged:: 0.7.0  Added ``*AttributeDocumenter``\s
+.. versionchanged:: 1.1.0  Added :class:`~.SlotsAttributeDocumenter`
+.. versionchanged:: 3.7.0  Added :class:`~.PropertyDocumenter`
 
-.. versionchanged:: 0.7.0
-
-	Added ``*AttributeDocumenter``\s
-
-.. versionchanged:: 1.1.0
-
-	Added :class:`~.SlotsAttributeDocumenter`
 
 Usage
 ----------
@@ -45,13 +40,13 @@ Usage
 
 		Show this instead of the type taken from the Python source code.
 
-
+	An example of the output cen be seen below for :py:obj:`~.type_template`.
 
 API Reference
 ----------------
-"""  # noqa D400
+"""  # noqa: D400
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -101,40 +96,62 @@ API Reference
 #
 
 # stdlib
+import importlib
 import sys
-from typing import Any, List, get_type_hints
+import warnings
+from contextlib import suppress
+from typing import Any, List, Optional, Type, cast, get_type_hints
 
 # 3rd party
-import sphinx.ext.autodoc
+import sphinx
+from docutils.statemachine import StringList
 from sphinx.application import Sphinx
+from sphinx.errors import PycodeError
 from sphinx.ext.autodoc import (
 		INSTANCEATTR,
 		SLOTSATTR,
 		UNINITIALIZED_ATTR,
-		AttributeDocumenter,
+		ClassDocumenter,
 		ClassLevelDocumenter,
-		DataDocumenter,
+		DocstringStripSignatureMixin,
 		Documenter,
 		ModuleDocumenter,
 		ModuleLevelDocumenter,
-		Options
+		Options,
+		annotation_option,
+		import_object,
+		logger,
+		mock
 		)
 from sphinx.ext.autodoc.directive import DocumenterBridge
-from sphinx.util.inspect import ForwardRef, object_description, safe_getattr
+from sphinx.pycode import ModuleAnalyzer
+from sphinx.util import inspect
+from sphinx.util.inspect import ForwardRef, getdoc, object_description, safe_getattr
 
 # this package
-from sphinx_toolbox.more_autodoc.typehints import format_annotation
-from sphinx_toolbox.utils import SphinxExtMetadata, flag, metadata_add_version
+from sphinx_toolbox._data_documenter import DataDocumenter
+from sphinx_toolbox.more_autodoc import _documenter_add_content
+from sphinx_toolbox.more_autodoc.typehints import _resolve_forwardref, format_annotation
+from sphinx_toolbox.utils import (
+		RemovedInSphinx50Warning,
+		SphinxExtMetadata,
+		add_nbsp_substitution,
+		flag,
+		metadata_add_version,
+		prepare_docstring
+		)
 
-__all__ = [
+__all__ = (
 		"VariableDocumenter",
 		"TypedAttributeDocumenter",
 		"InstanceAttributeDocumenter",
+		"PropertyDocumenter",
 		"SlotsAttributeDocumenter",
 		"type_template",
+		"old_type_template",
 		"get_variable_type",
 		"setup",
-		]
+		)
 
 
 def get_variable_type(documenter: Documenter) -> str:
@@ -149,12 +166,8 @@ def get_variable_type(documenter: Documenter) -> str:
 	except NameError:
 		# Failed to evaluate ForwardRef (maybe TYPE_CHECKING)
 		annotations = safe_getattr(documenter.parent, "__annotations__", {})
-	except TypeError:
-		annotations = {}
-	except KeyError:
-		# a broken class found (refs: https://github.com/sphinx-doc/sphinx/issues/8084)
-		annotations = {}
-	except AttributeError:
+	except (TypeError, KeyError, AttributeError):
+		# KeyError: a broken class found (refs: https://github.com/sphinx-doc/sphinx/issues/8084)
 		# AttributeError is raised on 3.5.2 (fixed by 3.5.3)
 		annotations = {}
 
@@ -179,10 +192,9 @@ def get_variable_type(documenter: Documenter) -> str:
 				if annotation.isidentifier() and annotation in module_dict:
 					return format_annotation(module_dict[annotation])
 				else:
-					if sys.version_info < (3, 9):
-						return format_annotation(ForwardRef(annotation)._evaluate(module_dict, module_dict))
-					else:
-						return format_annotation(ForwardRef(annotation)._evaluate(module_dict, module_dict, set()))
+					ref = ForwardRef(annotation)
+					evaled = _resolve_forwardref(ref, documenter.parent.__module__)
+					return format_annotation(evaled)
 
 			except (NameError, TypeError, ValueError, AttributeError):
 				return annotation
@@ -190,15 +202,35 @@ def get_variable_type(documenter: Documenter) -> str:
 			return ''
 
 
-type_template = "   **Type:** |nbsp| |nbsp| |nbsp| |nbsp| %s"
+old_type_template = "   **Type:** |nbsp| |nbsp| |nbsp| |nbsp| %s"
 """
-Template for rendering type annotations in :class:`~.VariableDocumenter`,
+Old template for rendering type annotations in :class:`~.VariableDocumenter`,
 :class:`~.TypedAttributeDocumenter` and :class:`~.InstanceAttributeDocumenter`.
 
 Renders like:
 
 	**Type:** |nbsp| |nbsp| |nbsp| |nbsp| :class:`str`
 
+.. note::
+
+	Be sure to call :func:~.add_nbsp_substitution` in the ``setup`` function
+	of any extensions using this template.
+"""
+
+type_template = "   **Type:**    %s"
+"""
+Template for rendering type annotations in :class:`~.VariableDocumenter`,
+:class:`~.TypedAttributeDocumenter` and :class:`~.InstanceAttributeDocumenter`.
+
+Renders like:
+
+	**Type:** \u00A0\u00A0\u00A0\u00A0 :class:`str`
+
+.. versionchanged:: 3.4.0
+
+	This template now uses the unicode codepoint for a non-breaking space,
+	rather than the ReStructuredText substitution used in 3.3.0 and earlier.
+	The old template is available as ``old_type_template``.
 """
 
 
@@ -209,7 +241,10 @@ class VariableDocumenter(DataDocumenter):
 
 	directivetype = "data"
 	objtype = "variable"
-	priority: float = DataDocumenter.priority + 0.5  # type: ignore  # keeps it below TypeVarDocumenter
+
+	# keeps it below TypeVarDocumenter
+	priority: float = DataDocumenter.priority + 0.5  # type: ignore[assignment]
+
 	option_spec = {
 			"no-value": flag,
 			"no-type": flag,
@@ -221,8 +256,9 @@ class VariableDocumenter(DataDocumenter):
 	def __init__(self, directive: DocumenterBridge, name: str, indent: str = '') -> None:
 		super().__init__(directive=directive, name=name, indent=indent)
 		self.options = Options(self.options.copy())
+		add_nbsp_substitution(self.env.app.config)
 
-	def add_directive_header(self, sig: str):
+	def add_directive_header(self, sig: str) -> None:
 		"""
 		Add the directive's header.
 
@@ -241,12 +277,10 @@ class VariableDocumenter(DataDocumenter):
 				if "value" in self.options:
 					self.add_line(f"   :value: {self.options['value']}", sourcename)
 				else:
-					try:
+					with suppress(ValueError):
 						if self.object is not UNINITIALIZED_ATTR:
 							objrepr = object_description(self.object)
 							self.add_line(f"   :value: {objrepr}", sourcename)
-					except ValueError:
-						pass
 
 			self.add_line('', sourcename)
 
@@ -257,8 +291,13 @@ class VariableDocumenter(DataDocumenter):
 					# obtain type annotation for this data
 					the_type = get_variable_type(self)
 					if not the_type.strip():
+						obj_type = type(self.object)
+
+						if obj_type is object:
+							return
+
 						try:
-							the_type = format_annotation(type(self.object))
+							the_type = format_annotation(obj_type)
 						except Exception:
 							return
 
@@ -269,26 +308,112 @@ class VariableDocumenter(DataDocumenter):
 			super().add_directive_header(sig)
 
 
-class TypedAttributeDocumenter(AttributeDocumenter):
+class TypedAttributeDocumenter(DocstringStripSignatureMixin, ClassLevelDocumenter):  # type: ignore[misc]
 	"""
-	Alternative version of :class:`sphinx.ext.autodoc.AttributeDocumenter`
+	Alternative version of
+	:class:`autodoc.AttributeDocumenter <sphinx.ext.autodoc.AttributeDocumenter>`
 	with better type hint rendering.
 
 	Specialized Documenter subclass for attributes.
 
 	.. versionadded:: 0.7.0
+	.. versionchanged:: 1.0.0  Now uses the type of the variable if it is not explicitly annotated.
+	"""  # noqa: D400
 
-	.. versionchanged:: 1.0.0
+	objtype = "attribute"
+	member_order = 60
+	option_spec = dict(ModuleLevelDocumenter.option_spec)
+	option_spec["annotation"] = annotation_option
 
-		Now uses the type of the variable if it is not explicitly annotated.
-	"""  # noqa D400
+	# must be higher than the MethodDocumenter, else it will recognize
+	# some non-data descriptors as methods
+	priority = 10
 
 	def __init__(self, directive: DocumenterBridge, name: str, indent: str = '') -> None:
 		super().__init__(directive=directive, name=name, indent=indent)
 		self.options = Options(self.options.copy())
 		self._datadescriptor = True
 
-	def add_directive_header(self, sig: str):
+	@staticmethod
+	def is_function_or_method(obj: Any) -> bool:  # noqa: D102
+		return inspect.isfunction(obj) or inspect.isbuiltin(obj) or inspect.ismethod(obj)
+
+	@classmethod
+	def can_document_member(cls, member: Any, membername: str, isattr: bool, parent: Any) -> bool:
+		"""
+		Called to see if a member can be documented by this documenter.
+		"""
+
+		if inspect.isattributedescriptor(member):
+			return True
+		elif (
+				not isinstance(parent, ModuleDocumenter) and not inspect.isroutine(member)
+				and not isinstance(member, type)
+				):
+			return True
+		else:
+			return False
+
+	def document_members(self, all_members: bool = False) -> None:  # noqa: D102
+		pass
+
+	def isinstanceattribute(self) -> bool:
+		"""
+		Check the subject is an instance attribute.
+		"""
+
+		try:
+			analyzer = ModuleAnalyzer.for_module(self.modname)
+			attr_docs = analyzer.find_attr_docs()
+			if self.objpath:
+				key = ('.'.join(self.objpath[:-1]), self.objpath[-1])
+				if key in attr_docs:
+					return True
+
+			return False
+		except PycodeError:
+			return False
+
+	def import_object(self, raiseerror: bool = False) -> bool:
+		"""
+		Import the object given by *self.modname* and *self.objpath* and set it as ``self.object``.
+
+		:returns: :py:obj:`True` if successful, :py:obj:`False` if an error occurred.
+		"""
+
+		try:
+			ret = super().import_object(raiseerror=True)
+			if inspect.isenumattribute(self.object):
+				self.object = self.object.value
+			if inspect.isattributedescriptor(self.object):
+				self._datadescriptor = True
+			else:
+				# if it's not a data descriptor
+				self._datadescriptor = False
+		except ImportError as exc:
+			if self.isinstanceattribute():
+				self.object = INSTANCEATTR
+				self._datadescriptor = False
+				ret = True
+			elif raiseerror:
+				raise
+			else:
+				logger.warning(exc.args[0], type="autodoc", subtype="import_object")
+				self.env.note_reread()
+				ret = False
+
+		return ret
+
+	def get_real_modname(self) -> str:
+		"""
+		Get the real module name of an object to document.
+
+		It can differ from the name of the module through which the object was imported.
+		"""
+
+		return self.get_attr(self.parent or self.object, "__module__", None) or self.modname
+
+	def add_directive_header(self, sig: str) -> None:
 		"""
 		Add the directive's header.
 
@@ -308,12 +433,10 @@ class TypedAttributeDocumenter(AttributeDocumenter):
 				if "value" in self.options:
 					self.add_line("   :value: " + self.options["value"], sourcename)
 				else:
-					try:
+					with suppress(ValueError):
 						if self.object is not INSTANCEATTR:
 							objrepr = object_description(self.object)
 							self.add_line("   :value: " + objrepr, sourcename)
-					except ValueError:
-						pass
 
 			self.add_line('', sourcename)
 
@@ -324,8 +447,13 @@ class TypedAttributeDocumenter(AttributeDocumenter):
 					# obtain type annotation for this attribute
 					the_type = get_variable_type(self)
 					if not the_type.strip():
+						obj_type = type(self.object)
+
+						if obj_type is object:
+							return
+
 						try:
-							the_type = format_annotation(type(self.object))
+							the_type = format_annotation(obj_type)
 						except Exception:
 							return
 
@@ -335,24 +463,82 @@ class TypedAttributeDocumenter(AttributeDocumenter):
 		else:
 			super().add_directive_header(sig)
 
+	def get_doc(
+			self,
+			encoding: Optional[str] = None,
+			ignore: Optional[int] = None,
+			) -> List[List[str]]:
+		"""
+		Decode and return lines of the docstring(s) for the object.
+
+		:param encoding:
+		:param ignore:
+		"""
+
+		# Disable `autodoc_inherit_docstring` temporarily to avoid to obtain
+		# a docstring from the value which descriptor returns unexpectedly.
+		# ref: https://github.com/sphinx-doc/sphinx/issues/7805
+		orig = self.env.config.autodoc_inherit_docstrings
+
+		try:
+			self.env.config.autodoc_inherit_docstrings = False  # type: ignore[attr-defined]
+
+			# Sphinx's signature is wrong wrt Optional
+			if sphinx.version_info >= (4, 0):
+				if encoding is not None:
+					raise TypeError("The 'encoding' argument to get_doc was removed in Sphinx 4")
+
+				if self._new_docstrings is not None:
+					return self._new_docstrings or []
+
+				docstring = getdoc(
+						self.object,
+						self.get_attr,
+						self.config.autodoc_inherit_docstrings,
+						self.parent,
+						self.object_name,
+						)
+
+				if docstring:
+					tab_width = self.directive.state.document.settings.tab_width
+					return [prepare_docstring(docstring, ignore, tab_width)]
+				return []
+
+			else:
+				enc = cast(str, encoding)
+				ign = cast(int, ignore)
+				super_get_doc = super().get_doc(enc, ign)  # type: ignore[call-arg]
+				return super_get_doc or []
+		finally:
+			self.env.config.autodoc_inherit_docstrings = orig  # type: ignore[attr-defined]
+
+	def add_content(self, more_content: Optional[StringList], no_docstring: bool = False) -> None:
+		"""
+		Add content from docstrings, attribute documentation and user.
+		"""
+
+		if not self._datadescriptor:
+			# if it's not a data descriptor, its docstring is very probably the wrong thing to display
+			no_docstring = True
+
+		_documenter_add_content(self, more_content, no_docstring)
+
 
 class InstanceAttributeDocumenter(TypedAttributeDocumenter):
 	"""
-	Alternative version of :class:`sphinx.ext.autodoc.InstanceAttributeDocumenter`
+	Alternative version of
+	:class:`autodoc.InstanceAttributeDocumenter <sphinx.ext.autodoc.InstanceAttributeDocumenter>`
 	with better type hint rendering.
 
 	Specialized Documenter subclass for attributes that cannot be imported
 	because they are instance attributes (e.g. assigned in ``__init__``).
 
 	.. versionadded:: 0.7.0
+	.. versionchanged:: 1.0.0  Now uses the type of the variable if it is not explicitly annotated.
+	"""  # noqa: D400
 
-	.. versionchanged:: 1.0.0
-
-		Now uses the type of the variable if it is not explicitly annotated.
-	"""  # noqa D400
-
-	objtype = sphinx.ext.autodoc.InstanceAttributeDocumenter.objtype
-	directivetype = sphinx.ext.autodoc.InstanceAttributeDocumenter.directivetype
+	objtype = "instanceattribute"
+	directivetype = "attribute"
 	member_order = 60
 
 	# must be higher than TypedAttributeDocumenter
@@ -384,7 +570,14 @@ class InstanceAttributeDocumenter(TypedAttributeDocumenter):
 		Import and return the attribute's parent.
 		"""
 
-		return sphinx.ext.autodoc.InstanceAttributeDocumenter.import_parent(self)  # type: ignore
+		try:
+			parent = importlib.import_module(self.modname)
+			for name in self.objpath[:-1]:
+				parent = self.get_attr(parent, name)
+
+			return parent
+		except (ImportError, AttributeError):
+			return None
 
 	def import_object(self, raiseerror: bool = False) -> bool:
 		"""
@@ -393,12 +586,14 @@ class InstanceAttributeDocumenter(TypedAttributeDocumenter):
 		:param raiseerror:
 		"""
 
-		return sphinx.ext.autodoc.InstanceAttributeDocumenter.import_object(
-				self,  # type: ignore
-				raiseerror=raiseerror,
-				)
+		# disguise as an attribute
+		self.objtype = "attribute"
+		self.object = INSTANCEATTR
+		self.parent = self.import_parent()
+		self._datadescriptor = False
+		return True
 
-	def add_content(self, more_content: Any, no_docstring: bool = False):
+	def add_content(self, more_content: Any, no_docstring: bool = False) -> None:
 		"""
 		Never try to get a docstring from the object.
 		"""
@@ -407,18 +602,22 @@ class InstanceAttributeDocumenter(TypedAttributeDocumenter):
 
 
 class SlotsAttributeDocumenter(TypedAttributeDocumenter):
-	"""
-	Alternative version of :class:`sphinx.ext.autodoc.InstanceAttributeDocumenter`
+	r"""
+	Alternative version of
+	:class:`autodoc.InstanceAttributeDocumenter <sphinx.ext.autodoc.InstanceAttributeDocumenter>`
 	with better type hint rendering.
 
 	Specialized Documenter subclass for attributes that cannot be imported
 	because they are attributes in __slots__.
 
 	.. versionadded:: 1.1.0
-	"""  # noqa D400
 
-	objtype = sphinx.ext.autodoc.SlotsAttributeDocumenter.objtype
-	directivetype = sphinx.ext.autodoc.SlotsAttributeDocumenter.directivetype
+	.. latex:vspace:: 10px
+
+	"""  # noqa: D400
+
+	objtype = "slotsattribute"
+	directivetype = "attribute"
 	member_order = 60
 
 	# must be higher than AttributeDocumenter
@@ -452,12 +651,34 @@ class SlotsAttributeDocumenter(TypedAttributeDocumenter):
 		:param raiseerror:
 		"""
 
-		return sphinx.ext.autodoc.SlotsAttributeDocumenter.import_object(
-				self,  # type: ignore
-				raiseerror=raiseerror,
-				)
+		# disguise as an attribute
+		self.objtype = "attribute"
+		self._datadescriptor = True
 
-	def get_doc(self, encoding: str = None, ignore: int = None) -> List[List[str]]:
+		with mock(self.env.config.autodoc_mock_imports):
+			try:
+				ret = import_object(
+						self.modname,
+						self.objpath[:-1],
+						"class",
+						attrgetter=self.get_attr,
+						warningiserror=self.env.config.autodoc_warningiserror
+						)
+				self.module, _, _, self.parent = ret
+				return True
+			except ImportError as exc:
+				if raiseerror:
+					raise
+				else:
+					logger.warning(exc.args[0], type="autodoc", subtype="import_object")
+					self.env.note_reread()
+					return False
+
+	def get_doc(
+			self,
+			encoding: Optional[str] = None,
+			ignore: Optional[int] = None,
+			) -> List[List[str]]:
 		"""
 		Decode and return lines of the docstring(s) for the object.
 
@@ -465,11 +686,93 @@ class SlotsAttributeDocumenter(TypedAttributeDocumenter):
 		:param ignore:
 		"""
 
-		return sphinx.ext.autodoc.SlotsAttributeDocumenter.get_doc(
-				self,  # type: ignore
-				encoding=encoding,
-				ignore=ignore,
-				)
+		if sphinx.version_info >= (4, 0):
+			if encoding is not None:
+				raise TypeError("The 'encoding' argument to get_doc was removed in Sphinx 4")
+
+		if ignore is not None:
+			if sphinx.version_info >= (5, 0):
+				raise TypeError("The 'ignore' argument to get_doc was removed in Sphinx 5")
+			else:
+				warnings.warn(
+						"The 'ignore' argument to get_doc() is deprecated.",
+						RemovedInSphinx50Warning,  # type: ignore[arg-type]
+						stacklevel=2
+						)
+
+		name = self.objpath[-1]
+		__slots__ = safe_getattr(self.parent, "__slots__", [])
+
+		if isinstance(__slots__, dict) and isinstance(__slots__.get(name), str):
+			return [prepare_docstring(__slots__[name])]
+		else:
+			return []
+
+
+class PropertyDocumenter(TypedAttributeDocumenter):
+	"""
+	Specialized Documenter subclass for properties.
+
+	.. versionadded:: 3.7.0
+	"""
+
+	objtype = "property"
+	member_order = 60
+	priority = 11
+
+	if sphinx.version_info[0] < 4:
+		directivetype = "method"
+
+	@classmethod
+	def can_document_member(
+			cls: Type[Documenter],
+			member: Any,
+			membername: str,
+			isattr: bool,
+			parent: Any,
+			) -> bool:
+		"""
+		Called to see if a member can be documented by this documenter.
+
+		This documenter only documents SLOTSATTR members.
+
+		:param member: The member being checked.
+		:param membername: The name of the member.
+		:param isattr:
+		:param parent: The parent of the member.
+		"""
+
+		return inspect.isproperty(member) and isinstance(parent, ClassDocumenter)
+
+	def add_directive_header(self, sig: str) -> None:
+		"""
+		Add the directive's header.
+
+		:param sig:
+		"""
+
+		sourcename = self.get_sourcename()
+		no_value = self.options.get("no-value", False)
+
+		ClassLevelDocumenter.add_directive_header(self, sig)
+		if inspect.isabstractmethod(self.object):
+			self.add_line("   :abstractmethod:", sourcename)
+
+		if sphinx.version_info[0] < 4:
+			self.add_line("   :property:", self.get_sourcename())
+
+		if not self.options.get("annotation", ''):
+			# data descriptors do not have useful values
+			if not no_value and not self._datadescriptor:
+				if "value" in self.options:
+					self.add_line("   :value: " + self.options["value"], sourcename)
+				else:
+					with suppress(ValueError):
+						if self.object is not INSTANCEATTR:
+							objrepr = object_description(self.object)
+							self.add_line("   :value: " + objrepr, sourcename)
+
+			self.add_line('', sourcename)
 
 
 @metadata_add_version
@@ -477,13 +780,18 @@ def setup(app: Sphinx) -> SphinxExtMetadata:
 	"""
 	Setup :mod:`sphinx_toolbox.more_autodoc.variables`.
 
-	:param app: The Sphinx app.
+	:param app: The Sphinx application.
 	"""
 
 	app.setup_extension("sphinx.ext.autodoc")
+	app.setup_extension("sphinx_toolbox.more_autosummary")
+
 	app.add_autodocumenter(VariableDocumenter)
 	app.add_autodocumenter(TypedAttributeDocumenter, override=True)
 	app.add_autodocumenter(InstanceAttributeDocumenter, override=True)
 	app.add_autodocumenter(SlotsAttributeDocumenter, override=True)
+	app.add_autodocumenter(PropertyDocumenter, override=True)
+
+	app.connect("config-inited", lambda _, config: add_nbsp_substitution(config))
 
 	return {"parallel_read_safe": True}
