@@ -1,6 +1,9 @@
+# stdlib
+from typing import Optional, cast
+
 # 3rd party
 import pytest
-from bs4 import BeautifulSoup  # type: ignore[import-untyped]
+from bs4 import BeautifulSoup, Tag
 from sphinx.application import Sphinx
 
 # this package
@@ -17,7 +20,9 @@ def test_build_github(gh_src_app: Sphinx):
 @pytest.mark.parametrize("github_source_page", ["index.html"], indirect=True)
 def test_output_github(github_source_page: BeautifulSoup, html_regression: HTMLRegressionFixture):
 	# Make sure the page title is what you expect
-	title = github_source_page.find("h1").contents[0].strip()
+	h1 = cast(Optional[Tag], github_source_page.find("h1"))
+	assert h1 is not None
+	title = cast(str, h1.contents[0]).strip()
 	assert "sphinx-toolbox Demo" == title
 
 	tag_count = 0
@@ -27,12 +32,12 @@ def test_output_github(github_source_page: BeautifulSoup, html_regression: HTMLR
 			if a_tag.contents[0] == "sphinx_toolbox/config.py":
 				assert a_tag[
 						"href"
-						] == "https://github.com/domdfcoding/sphinx-toolbox/blob/master/sphinx_toolbox/config.py"
+						] == "https://github.com/sphinx-toolbox/sphinx-toolbox/blob/master/sphinx_toolbox/config.py"
 		elif tag_count == 1:
 			if a_tag.contents[0] == "source code":
 				assert a_tag[
 						"href"
-						] == "https://github.com/domdfcoding/sphinx-toolbox/blob/master/sphinx_toolbox/config.py"
+						] == "https://github.com/sphinx-toolbox/sphinx-toolbox/blob/master/sphinx_toolbox/config.py"
 		tag_count += 1
 
 	assert tag_count == 2
